/*********************************************************************************
 *  Display_Graphics.c
 *  ------------------
 *
 *  Description
 *  Drawing graphic primitives
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

#include <avr/io.h> 
#include <util/delay.h> 


#include "Display_SPI.h"
#include "Display_Graphics.h"


/*********************************************************************************
 *  FillScreen_BrushColor
 *********************************************************************************
 *  Description: Fills screen with brush color
 *
 *  Input Arguments:
 *    none
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void FillScreen_BrushColor(void)
{

  if (!SendCommand(CMD_FillScreen_BrushColor)) return; // Send command

  SPI_RW(0xFF);     // Dummy Byte

  WaitWhileDisplayBusy();             // Wait while Display signals BUSY
  SPI_SS_HIGH                         // Deselect SPI
}

/*********************************************************************************
 *  FillScreen_PenColor
 *********************************************************************************
 *  Description: Fills screen with pen color
 *
 *  Input Arguments:
 *    none
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void FillScreen_PenColor(void)
{

  if (!SendCommand(CMD_FillScreen_PenColor)) return; // Send command

  SPI_RW(0xFF);     // Dummy Byte

  WaitWhileDisplayBusy();             // Wait while Display signals BUSY
  SPI_SS_HIGH                         // Deselect SPI

}


/*********************************************************************************
 *  FillScreen_RGBColor
 *********************************************************************************
 *  Description: Fills screen with color RGB
 *
 *  Input Arguments:
 *    r     : red color     (1Byte) 0..255
 *    g     : green color   (1Byte) 0..255
 *    b     : blue color    (1Byte) 0..255
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void FillScreen_RGBColor(uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_FillScreen_RGBColor)) return; // Send command

  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  WaitWhileDisplayBusy();             // Wait while Display signals BUSY
  SPI_SS_HIGH                         // Deselect SPI

}




/*********************************************************************************
 *  MoveTo
 *********************************************************************************
 *  Description: Moves graphical cursor to position x/y
 *
 *  Input Arguments:
 *    x	: location x	(2Byte)
 *    y	: location y	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void MoveTo(uint16_t x, uint16_t y)
{

  if (!SendCommand(CMD_MoveTo)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  LineTo_PenColor
 *********************************************************************************
 *  Description: Draws a line with pen color
 *               Start coordinate is defined by MoveTo command
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void LineTo_PenColor(uint16_t x, uint16_t y)
{

  if (!SendCommand(CMD_LineTo_PenColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  LineTo_BrushColor
 *********************************************************************************
 *  Description: Draws a line with brush color
 *               Start coordinate is defined by MoveTo command
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void LineTo_BrushColor(uint16_t x, uint16_t y)
{

  if (!SendCommand(CMD_LineTo_BrushColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  LineTo_RGBColor
 *********************************************************************************
 *  Description: Draws a line with RGB color
 *               Start coordinate is defined by MoveTo command
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void LineTo_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_LineTo_RGBColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawPixel_PenColor
 *********************************************************************************
 *  Description: Draws a pixel with pen color
 *
 *  Input Arguments:
 *    x	: location x	(2Byte)
 *    y	: location y	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawPixel_PenColor(uint16_t x, uint16_t y)
{

  if (!SendCommand(CMD_DrawPixel_PenColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawPixel_BrushColor
 *********************************************************************************
 *  Description: Draws a pixel with brush color
 *
 *  Input Arguments:
 *    x	: location x	(2Byte)
 *    y	: location y	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawPixel_BrushColor(uint16_t x, uint16_t y)
{

  if (!SendCommand(CMD_DrawPixel_BrushColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawPixel_RGBColor
 *********************************************************************************
 *  Description: Draws a pixel with RGB color
 *
 *  Input Arguments:
 *    x	: location x	(2Byte)
 *    y	: location y	(2Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawPixel_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_DrawPixel_RGBColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x1
  SPI_RW((uint8_t)x);          // LSB x1
  SPI_RW((uint8_t)(y>>8));     // MSB y1
  SPI_RW((uint8_t)y);          // LSB y1
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}




/*********************************************************************************
 *  DrawLine_PenColor
 *********************************************************************************
 *  Description: Draws a line with pen color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawLine_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawLine_PenColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawLine_BrushColor
 *********************************************************************************
 *  Description: Draws a line with brush color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawLine_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawLine_BrushColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawLine_RGBColor
 *********************************************************************************
 *  Description: Draws a line with RGB color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawLine_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_DrawLine_RGBColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawRect_PenColor
 *********************************************************************************
 *  Description: Draws a rectangle with pen color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawRect_PenColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawRect_PenColorFill
 *********************************************************************************
 *  Description: Draws a filled rectangle with pen color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_PenColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawRect_PenColorFill)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}


/*********************************************************************************
 *  DrawRect_BrushColor
 *********************************************************************************
 *  Description: Draws a rectangle with brush color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawRect_BrushColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawRect_BrushColorFill
 *********************************************************************************
 *  Description: Draws a filled rectangle with brush color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_BrushColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{

  if (!SendCommand(CMD_DrawRect_BrushColorFill)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawRect_RGBColor
 *********************************************************************************
 *  Description: Draws a rectangle with RGB color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_DrawRect_RGBColor)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawRect_RGBColorFill
 *********************************************************************************
 *  Description: Draws a filled rectangle with RGB color
 *
 *  Input Arguments:
 *    x1	: location x1	(2Byte)
 *    y1	: location y1	(2Byte)
 *    x2	: location x2	(2Byte)
 *    y2	: location y2	(2Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawRect_RGBColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_DrawRect_RGBColorFill)) return; // Send command

  SPI_RW((uint8_t)(x1>>8));     // MSB x1
  SPI_RW((uint8_t)x1);          // LSB x1
  SPI_RW((uint8_t)(y1>>8));     // MSB y1
  SPI_RW((uint8_t)y1);          // LSB y1
  SPI_RW((uint8_t)(x2>>8));     // MSB x2
  SPI_RW((uint8_t)x2);          // LSB x2
  SPI_RW((uint8_t)(y2>>8));     // MSB y2
  SPI_RW((uint8_t)y2);          // LSB y2
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawCircle_PenColor
 *********************************************************************************
 *  Description: Draws a circle with pen color
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *    radius: radius		(1Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawCircle_PenColor(uint16_t x, uint16_t y, uint8_t radius)
{

  if (!SendCommand(CMD_DrawCircle_PenColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y
  SPI_RW(radius);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawCircle_BrushColor
 *********************************************************************************
 *  Description: Draws a circle with brush color
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *    radius: radius		(1Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawCircle_BrushColor(uint16_t x, uint16_t y, uint8_t radius)
{

  if (!SendCommand(CMD_DrawCircle_BrushColor)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y
  SPI_RW(radius);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  DrawCircle_RGBColor
 *********************************************************************************
 *  Description: Draws a circle with RGB color
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *    radius: radius		(1Byte)
 *    r     : red color     (1Byte) 0..63
 *    g     : green color   (1Byte) 0..63
 *    b     : blue color    (1Byte) 0..63
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawCircle_RGBColor(uint16_t x, uint16_t y, uint8_t radius, uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_DrawCircle_RGBColor))
  {
  
 PORTE &= ~(1<<PE0);
 while(1) {};
   return; // Send command
  }
  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x
  SPI_RW((uint8_t)(y>>8));     // MSB y
  SPI_RW((uint8_t)y);          // LSB y
  SPI_RW(radius);
  SPI_RW(r);
  SPI_RW(g);
  SPI_RW(b);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}




/*********************************************************************************
 *  DrawImage_Begin
 *********************************************************************************
 *  Description: Displays image data in rectangle area defined by
 *               coordinate x/y, width and height. Display area is filled
 *               row by row, column is auto incremented.
 *               A pixel consists of three color bytes r, g and b. Please note
 *               that color depth is 6bit. Possibly you have to adjust the
 *               color depth of your image data (in that case shift color data).
 *               Note: x+width < display width
 *                     y+height < display height
 *
 *               (1) Call DrawImage_Begin(x, y, width, height) to start data transfer
 *               (2) Send RGB Pixel Data
 *               (3) Call DrawImage_End of deselect SPI (CS high) to complete data transfer
 *
 *  Input Arguments:
 *    x	: location x	(2Byte)
 *    y	: location y	(2Byte)
 *    width	: width of image in pixels	(2Byte)
 *    height: height of image in pixels	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DrawImage_Begin(uint16_t x, uint16_t y, uint16_t width, uint16_t height)
{

  if (!SendCommand(CMD_DrawImage_Begin)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x1
  SPI_RW((uint8_t)x);          // LSB x1
  SPI_RW((uint8_t)(y>>8));     // MSB y1
  SPI_RW((uint8_t)y);          // LSB y1

  SPI_RW((uint8_t)(width>>8)); // MSB width
  SPI_RW((uint8_t)width);      // LSB width
  SPI_RW((uint8_t)(height>>8));// MSB height
  SPI_RW((uint8_t)height);     // LSB height

  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

void DrawImage_WritePixel(uint8_t r, uint8_t g, uint8_t b)
{

  SPI_RW(r);     // R
  SPI_RW(g);     // G
  SPI_RW(b);     // B
  
}

void DrawImage_Stop(void)
{

  SPI_SS_HIGH                         // Deselect SPI

}
