

#include <avr/io.h> 
#include <util/delay.h> 
#include "Display_SPI.h"

//#define SPI_SOFTWARESPI


/*********************************************************************************
 *  SPI Interface
 *********************************************************************************
 *  Microcontroller: AVR ATmega128
 *
 *  SCK   : PB1  (OUTPUT)
 *  MOSI  : PB2  (OUTPUT)
 *  MISO  : PB3  (INPUT)
 *  CS_   : PB0  (OUTPUT)
 *
 *  BUSY_ : PB4 (INPUT, Pullup Resistor activated)
 *  TINT_ : PB5 (INPUT, Pullup Resistor activated)
 *
 *********************************************************************************/
 

#define SPI_MOSI_SET() PORTB |=  (1<<PB2);
#define SPI_MOSI_CLR() PORTB &=  ~(1<<PB2);

#define SPI_CLK_SET() PORTB |=  (1<<PB1)
#define SPI_CLK_CLR() PORTB &=  ~(1<<PB1)

#define SPI_MISO_IS_SET() PINB & (1<<PB3)


//********************************************************************************
// Software SPI
// fck max: 2MHz
//********************************************************************************

#ifdef SPI_SOFTWARESPI

void SPI_Init(void)
{
  // Port und Direction

  DDRB &= ~(1<<PINB4);   // Busy Input Pullup
  PORTB |= (1<<PB4);

  // SS\    PB0   OUTPUT
  // SCK    PB1   OUTPUT
  // MOSI   PB2   OUTPUT
  // MISO   PB3   INPUT

  PORTB |= (1<<PB2) | (1<<PB1) | (1<<PB0);  // High
  DDRB  |= (1<<PB2) | (1<<PB1) | (1<<PB0);  // Direction Output
  DDRB &= ~(1<<PB3);   // MISO Input

  // TOUCH_
  PORTB |= (1<<PB5);  // PINB5 Input mit Pullup  
  DDRB &= ~(1<<PB5);

}


uint8_t SPI_RW (uint8_t output)
{
  uint8_t data_in;
  uint8_t msk;
  uint8_t len; 
    
  msk = 0b10000000;
  len = 8; 
  data_in = 0; 

  while(len)   
  {   

   /*******************************************
    * Daten ausgeben
	*******************************************/
    SPI_CLK_CLR();

    if (output & msk)
	{
      SPI_MOSI_SET();
	} else {
      SPI_MOSI_CLR();
	}

	nop();

    SPI_CLK_SET();

	nop();

   /*******************************************
    * Daten einlesen
	*******************************************/	
	if(SPI_MISO_IS_SET())   
      data_in |= msk;   	
    	
	  	    
	msk >>= 1;
    len--;   

  }  
   
  return data_in;   

}		
}		


#else


//********************************************************************************
// AVR Hardware SPI
// fck max: 2MHz
//********************************************************************************


void SPI_Init (void)
{
  uint8_t data;

  // Port und Direction

  DDRB &= ~(1<<PINB4);   // Busy Input Pullup
  PORTB |= (1<<PB4);


  // SS\    PB0   OUTPUT
  // SCK    PB1   OUTPUT
  // MOSI   PB2   OUTPUT
  // MISO   PB3   INPUT

  PORTB |= (1<<PB2) | (1<<PB1) | (1<<PB0);  // High
  DDRB  |= (1<<PB2) | (1<<PB1) | (1<<PB0);  // Direction Output
  DDRB &= ~(1<<PB3);   // MISO Input

  // SPI Init
  SPSR = (1<<SPI2X);    //SPI double speed settings
  SPCR = (1<<SPE) | (1<<MSTR) | (1<<CPHA) | (1<<CPOL) | (1<<SPR0);// | (1<<SPR1) | (1<<SPR0);	//Enable SPI in Master mode, mode 3, Fosc/2

  data = SPSR;
  data = SPDR;
	

  // TOUCH_
  PORTB |= (1<<PB5);  // PINB5 Input mit Pullup  
  DDRB &= ~(1<<PB5);

}  


unsigned char SPI_RW (unsigned char output)
{
  unsigned char input;
	
  SPDR = output;							//put byte 'output' in SPI data register
  while(!(SPSR & 0x80));					//wait for transfer complete, poll SPIF-flag
  input = SPDR;							//read value in SPI data reg.
	
  return input;							//return the byte clocked in from SPI slave
}		

#endif















uint8_t WaitWhileDisplayBusy_Timeout500ms(void)
{
  uint16_t time;

  time = 50000;					// timeout 500ms
  while (DISPLAY_BUSY)
  {
    time--;
	if (0 == time)
	{
      // Timeout, deselect display spi
	  SPI_SS_HIGH						// timeout
      _delay_us(1);
	  return 0x00;
    }
    _delay_us(10);
  }

  return 0xFF;
}



uint8_t WaitWhileDisplayBusy(void)
{
  uint16_t time;

  time = 10000;					// timeout 50ms

  time = 50000;
  while (DISPLAY_BUSY)
  {
    time--;
	if (0 == time)
	{
      // Timeout, deselect display spi
	  SPI_SS_HIGH						// timeout
      _delay_us(1);
	  return 0x00;
    }
    _delay_us(5);
  }

  return 0xFF;
}


uint8_t WaitUntilDisplayBusy(void)
{
  uint16_t time;

  time = 10000;					// timeout 50ms

  time = 50000;
  while (DISPLAY_NOTBUSY)
  {
    time--;
	if (0 == time)
	{
      // Timeout, deselect display spi
	  SPI_SS_HIGH						// timeout
      _delay_us(1);
	  return 0x00;
    }
    _delay_us(5);
  }

  return 0xFF;
}

uint8_t WaitWhileTouch(void)
{
  uint16_t time;

  time = 10000;					// timeout 10ms

  while (TOUCHPANEL_EVENT)
  {
    time--;
	if (0 == time)
	{
      // Timeout, deselect display spi
	  SPI_SS_HIGH						// timeout
      _delay_us(1);
	  return 0x00;
    }
    _delay_us(1);
  }

  return 0xFF;
}



uint8_t SendCommand(uint16_t command)
{

 /**************************************
  Wait for display ready
  **************************************/

  if (!WaitWhileDisplayBusy()) return 0;  // timeout

 /**************************************
  Select display spi
  **************************************/
  SPI_SS_LOW
  _delay_us(2);

 /**************************************
  Send command to display
  **************************************/
  SPI_RW((uint8_t)(command>>8));  // MSB
  SPI_RW((uint8_t)command);      // LSB

 /**************************************
  Wait until display signals busy
  **************************************/
  if (!WaitUntilDisplayBusy()) return 0; // timeout

  return 0xFF;

}



