/*********************************************************************************
 *  Display_Text.c
 *  --------------
 *
 *  Description
 *  Text routines
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

#include <avr/io.h> 
#include <util/delay.h> 
#include <avr/pgmspace.h>

#include "Display_SPI.h"
#include "Display_Text.h"



/*********************************************************************************
 *  Display_Text
 *********************************************************************************
 *  Description: Displays a text
 *
 *  Note: If length of string exceeds 100 characters, string will be shortened to
 *        100 characters
 *
 *  Input Arguments:
 *    x		: location x	(2Byte)
 *    y		: location y	(2Byte)
 *    s     : pointer to string
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void DisplayText(uint16_t x, uint16_t y, unsigned char *s)
{
  uint8_t i;

  if (!SendCommand(CMD_DisplayText)) return;

  SPI_RW((uint8_t)(x>>8));     // MSB
  SPI_RW((uint8_t)x);          // LSB
  SPI_RW((uint8_t)(y>>8));     // MSB
  SPI_RW((uint8_t)y);          // LSB

  i = 0;
  while (s[i] && (i<100))      // max. string length = 100
  {
    SPI_RW(s[i]); 
    i++; 
  }
  SPI_RW(0); 

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

void DisplayText_F(uint16_t x, uint16_t y, char *s)
{
  uint8_t i;
  char c;

  if (!SendCommand(CMD_DisplayText)) return;

  SPI_RW((uint8_t)(x>>8));     // MSB
  SPI_RW((uint8_t)x);          // LSB
  SPI_RW((uint8_t)(y>>8));     // MSB
  SPI_RW((uint8_t)y);          // LSB

  c = pgm_read_byte(&(*s++));
  i = 0;
  while (c && (i<100))      // max. string length = 100
  {
    SPI_RW(c); 
    i++; 
    c = pgm_read_byte(&(*s++));
  }
  SPI_RW(0); 

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}



/*********************************************************************************
 *  SetFont
 *********************************************************************************
 *  Description: Set default textfont and numberfont
 *
 *  Note: 
 *
 *  Input Arguments:
 *    index : index of font (1Byte) 0..n-1, n = number of fonts
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void SetFont(uint8_t index)
{

  if (!SendCommand(CMD_SetFont)) return; // Send command

  SPI_RW(index);     // index of font

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  SetTextFont
 *********************************************************************************
 *  Description: Set default textfont
 *
 *  Note: 
 *
 *  Input Arguments:
 *    index : index of font (1Byte) 0..n-1, n = number of fonts
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void SetTextFont(uint8_t index)
{

  if (!SendCommand(CMD_SetTextFont)) return; // Send command

  SPI_RW(index);     // index of font

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}


/*********************************************************************************
 *  SetTextFont
 *********************************************************************************
 *  Description: Set default numberfont
 *
 *  Note: 
 *
 *  Input Arguments:
 *    index : index of font (1Byte) 0..n-1, n = number of fonts
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void SetNumberFont(uint8_t index)
{

  if (!SendCommand(CMD_SetNumberFont)) return; // Send command

  SPI_RW(index);     // index of font

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}




/*********************************************************************************
 *  SetTextMarginX
 *********************************************************************************
 *  Description: Set horizontal absolute limit (px) for text
 *
 *  Note: x=0 disables x-margin
 *
 *  Input Arguments:
 *    x		: location x of margin	(2Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void SetTextMarginX(uint16_t x)
{

  if (!SendCommand(CMD_SetTextMarginX)) return; // Send command

  SPI_RW((uint8_t)(x>>8));     // MSB x
  SPI_RW((uint8_t)x);          // LSB x

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}




/*********************************************************************************
 *  SetPenColor
 *********************************************************************************
 *  Description: Set Pen Color
 *
 *
 *  Input Arguments:
 *    r		: red color	  (1Byte) 0..63, 6bit
 *    g		: green color (1Byte) 0..63, 6bit
 *    b     : blue color  (1Byte) 0..63, 6bit
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 16.03.2011   State: tested | released
 *********************************************************************************/
void SetPenColor(uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_SetPenColor)) return;

  SPI_RW(r);    // red
  SPI_RW(g);    // green
  SPI_RW(b);    // blue

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  SetBrushColor
 *********************************************************************************
 *  Description: Set Brush Color
 *
 *
 *  Input Arguments:
 *    r		: red color	  (1Byte) 0..63, 6bit
 *    g		: green color (1Byte) 0..63, 6bit
 *    b     : blue color  (1Byte) 0..63, 6bit
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 16.03.2011   State: tested | released
 *********************************************************************************/
void SetBrushColor(uint8_t r, uint8_t g, uint8_t b)
{

  if (!SendCommand(CMD_SetBrushColor)) return;

  SPI_RW(r);    // red
  SPI_RW(g);    // green
  SPI_RW(b);    // blue

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}








