/*********************************************************************************
 *  Display_TouchPanel.c
 *  -------------------
 *
 *  Description
 *  Touchpanel related commands
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

#include <avr/io.h> 
#include <util/delay.h> 


#include "Display_SPI.h"
#include "Display_TouchPanel.h"

/*********************************************************************************
 *  TouchPanel_SetConfig
 *********************************************************************************
 *  Description: Set event configuration of touch panel
 *
 *  config bit position
 *  Bit 0: Enable TouchDown Event
 *  Bit 1: Enable TouchUp Event
 *  Bit 2: Enable TouchContinuous Event
 *
 *  due to future compatibility, clear all other bitpositions
 *
 *  Example:
 *  -------
 *    TouchPanel_SetConfig(ENABLE_TDown | ENABLE_TUp | ENABLE_TCont); enables all events
 *    equal to
 *    TouchPanel_SetConfig(0b00000111);
 *
 *  Input Arguments:
 *    config: (1Byte)
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/
void TouchPanel_SetConfig(uint8_t config)
{

  if (!SendCommand(CMD_TouchPanel_SetConfig)) return; // Send command

  SPI_RW(config);

  SPI_SS_HIGH                         // Deselect SPI
  WaitWhileDisplayBusy();             // Wait while Display signals BUSY

}

/*********************************************************************************
 *  TouchPanel_ReadEvent
 *********************************************************************************
 *  Description: Read touch panel event
 *
 *  TouchEvent bit position
 *  --------------------------------------------
 *  Bit 0: TouchDown Event
 *  Bit 1: TouchUp Event
 *  Bit 2: TouchContinuous Event
 *  Bit 3: Button Event 		(visual component)
 *  Bit 4: CheckBox Event 		(visual component)
 *  Bit 5: RadioGroup Event 	(visual component)
 *
 *  Note: Execution of TouchPanel_ReadEvent clears TOUCHINT signal of display
 *        module (set to logical 1)
 *
 *  Input Arguments:
 *    none
 *
 *  Output Variable
 *    event: (1Byte)
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

uint8_t TouchPanel_ReadEvent(void)
{
  uint8_t event;

  if (!SendCommand(CMD_TouchPanel_ReadEvent)) return 0; // Send command


  event = SPI_RW(0xFF);
  

 // while (TOUCHPANEL_EVENT) {  }  // wait while display signals touchevent

  if (!WaitWhileTouch()) return 0;

  SPI_SS_HIGH                           // Deselect SPI
  WaitWhileDisplayBusy();               // Wait while Display signals BUSY

  return event;
}


/*********************************************************************************
 *  TouchPanel_Calibrate
 *********************************************************************************
 *  Description: Start touch panel calibration procedure
 *
 *
 *  Input Arguments:
 *    none
 *
 *  Output Variable
 *    none
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

void TouchPanel_Calibrate(void)
{

  if (!SendCommand(CMD_TouchPanel_Calibrate)) return; // Send command

  // Touch Panel now shows Calibration Window
  // Display will signals BUSY high, if calibration process is done

  SPI_RW(0xFF);  // Dummy

  SPI_SS_HIGH                           // Deselect SPI
  WaitWhileDisplayBusy();               // Wait while Display signals BUSY

}


/*********************************************************************************
 *  TouchPanel_ReadCont
 *********************************************************************************
 *  Description: Read actual touch continual coordinate
 *
 *  Note: First you have to examine touch panel event. If an TouchContinual event
 *        occoured, coordinate is valid.
 *        If command execution fails, routine returns 0
 *
 *  Input Arguments:
 *    *x : pointer to x variable (x is a 16bit variable)
 *    *y : pointer to y variable (y is a 16bit variable)
 *
 *  Output Variable
 *    result: 1=successful, 0=execution command fails
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

uint8_t TouchPanel_ReadCont(uint16_t *x, uint16_t *y)
{
  uint16_t data;

  if (!SendCommand(CMD_TouchPanel_ReadCont)) return 0;  // execution fails


  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB x
  data |= SPI_RW(0xFF);              // LSB x

  *x = data;

  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB y
  data |= SPI_RW(0xFF);              // LSB y

  *y = data;

  SPI_SS_HIGH                           // Deselect SPI
  WaitWhileDisplayBusy();               // Wait while Display signals BUSY

  return 1;  // successfull

}

/*********************************************************************************
 *  TouchPanel_ReadDown
 *********************************************************************************
 *  Description: Read actual touch down coordinate
 *
 *  Note: First you have to examine touch panel event. If an TouchDown event
 *        occoured, coordinate is valid.
 *        If command execution fails, routine returns 0        
 *
 *  Input Arguments:
 *    *x : pointer to x variable (x is a 16bit variable)
 *    *y : pointer to y variable (y is a 16bit variable)
 *
 *  Output Variable
 *    result: 1=successful, 0=execution command fails
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

uint8_t TouchPanel_ReadDown(uint16_t *x, uint16_t *y)
{
  uint16_t data;

  if (!SendCommand(CMD_TouchPanel_ReadDown)) return 0; // execution fails

  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB x
  data |= SPI_RW(0xFF);              // LSB x

  *x = data;

  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB y
  data |= SPI_RW(0xFF);              // LSB y

  *y = data;

  SPI_SS_HIGH                           // Deselect SPI
  WaitWhileDisplayBusy();               // Wait while Display signals BUSY

  return 1;
}

/*********************************************************************************
 *  TouchPanel_ReadUp
 *********************************************************************************
 *  Description: Read actual touch up coordinate
 *
 *  Note: First you have to examine touch panel event. If an TouchUp event
 *        occoured, coordinate is valid.
 *        If command execution fails, routine returns 0        
 *
 *  Input Arguments:
 *    *x : pointer to x variable (x is a 16bit variable)
 *    *y : pointer to y variable (y is a 16bit variable)
 *
 *  Output Variable
 *    result: 1=successful, 0=execution command fails
 *
 *********************************************************************************
 *  Revision: 1   Date: 23.02.2011   State: tested | released
 *********************************************************************************/

uint8_t TouchPanel_ReadUp(uint16_t *x, uint16_t *y)
{
  uint16_t data;

  if (!SendCommand(CMD_TouchPanel_ReadUp)) return 0; // execution fails

  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB x
  data |= SPI_RW(0xFF);              // LSB x

  *x = data;

  data = (uint16_t)SPI_RW(0xFF)<<8;  // MSB y
  data |= SPI_RW(0xFF);              // LSB y

  *y = data;

  SPI_SS_HIGH                           // Deselect SPI
  WaitWhileDisplayBusy();               // Wait while Display signals BUSY

  return 1;
}





